# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: skip-file
# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RepositoryKeyRequestProperties(Model):
    """RepositoryKeyRequestProperties.

    :param id:
    :type id: str
    :param user_role: Possible values include: 'Reader', 'Contributor',
     'Admin'
    :type user_role: str or
     ~digitaltwinrepositoryprovisioningservice.models.enum
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'user_role': {'key': 'userRole', 'type': 'str'},
    }

    def __init__(self, id=None, user_role=None):
        super(RepositoryKeyRequestProperties, self).__init__()
        self.id = id
        self.user_role = user_role
